
/**
 * Na programovaní textovej adventúry sa dá pekne predviesť podstata
 * objektového modelovania. Budeme pracovať s bežnými pojmami z reálneho
 * sveta, ale namiesto ich reálneho zobrazovania budeme pracovať s textovými
 * opismi, čo je charakteristické pre tento druh hry.
 * 
 * Na ukážku je v tomto projekte vytvorených niekoľko jednoduchých tried
 * reprezentujúcich miestnosti, predmety a hráča (avatara).
 * 
 * Pri objektovom modelovaní ide o to, aby:
 *   – vlastnosti boli vyjadrené premennými, ktoré sú súkromné,
 *   – ku každej vlastnosti bolo poskytnuté programátorské rozhranie, to jest
 *     definícia jednej alebo viacerých verejných metód, ktoré slúžia na
 *     prácu s touto vlastnosťou,
 *   – rozrhanie sa môže skladať z troch základných „druhov“ metód:
 *      – metódy na zápis vlastností,
 *      – metódy na čítanie vlastností,
 *      – metódy na „inú prácu“ s vlastnosťami (napr. použitie)…
 * 
 * Každá trieda sa stará výhradne o svoje „životné funkcie“. Vytvorené
 * inštancie tried „oživujú“ hru. Používajú na to programátorské rozhranie
 * tried, to jest ich verejné metódy.
 */

public class Adventúra extends GRobot
{
	private final static Písmo písmoKonzoly =
		new Písmo("Verdana", Písmo.NORMÁLNE, 14);

	// Definície miestností:
	private final Miestnosť škola = new Miestnosť("škola", "Si v škole.");
	private final Miestnosť ulica = new Miestnosť("ulica", "Si na ulici.");
	private final Miestnosť park = new Miestnosť("park", "Si v parku.");

	// Definície predmetov:
	private final Predmet kniha = new Predmet("kniha", "Toto je kniha.");
	private final Predmet lopta = new Predmet("lopta", "Toto je lopta.");
	private final Predmet list = new Predmet("list", "Toto je list.");

	// Definícia avatara:
	private final Avatar avatar = new Avatar(škola);


	// Konštruktor
	public Adventúra()
	{
		// Priradenie vlastností jednotlivým inštanciám miestností.
		//    V tomto štádiu rozprogramovania to súvisí najmä s prepojením
		//    miestostí navzájom. Názov a opis miestnosti sú priradené priamo
		//    v pri konštrukcii (to jest pri new Miestnosť("…", "…") – pozri
		//    vyššie). Okrem toho tu môžu byť uvedené rôzne prepojenia –
		//    napríklad sú tu vložené predmety do jednotlivých miestností.

		// Škola:
		škola.prepojMaS(ulica);
		škola.vložPredmet(kniha);

		// Ulica:
		ulica.prepojMaS(škola);
		ulica.prepojMaS(park);
		ulica.vložPredmet(lopta);

		// Park:
		park.prepojMaS(ulica);
		park.vložPredmet(list);

		// Definícia obsluhy udalostí s jedinou reakciou spracúvajúcou
		// potvrdenie údajov v príkazovom riadku.
		new ObsluhaUdalostí()
		{
			public void potvrdenieÚdajov() { spracujPríkaz(); }
		};

		// 
		svet.písmo(písmoKonzoly);

		// Zobrazenie a aktivácia vstupného riadka.
		svet.neskrývajVstupnýRiadok();
		svet.začniVstup("Zadaj príkaz:");

		// Výpis počiatočného stavu hry.
		vypíšStav();
	}

	/**
	 * Výpis aktuálneho stavu hry.
	 */
	public void vypíšStav()
	{
		// Vypíše sa opis miestnosti, v ktorej sa práve nachádza avatar.
		avatar.kdeSom().vypíšOpis();

		// Vypíše sa zoznam dverí.
		avatar.kdeSom().vypíšZoznamDverí();

		// Výpis predmetov prítomných/viditeľných v miestnosti.
		avatar.kdeSom().vypíšZoznamPredmetov();

		// Výpis zoznamu predmetov v inventári avatara.
		avatar.vypíšInventár();

		// Vynechanie prázdneho riadka.
		svet.vypíšRiadok();
	}


	/**
	 * Pokus o prechod do miestnosti identifikovanej zadaným názvom. Ak
	 * metóda uspeje, vráti true, inak vráti false.
	 */
	public boolean choďDo(String názovMiestnosti)
	{
		// Hľadajú sa dvere:
		Miestnosť prechod = avatar.kdeSom().hľadajDvere(názovMiestnosti);

		if (null != prechod)
		{
			// Zmena miestnosti a vrátenie hodnoty true, ktorá v tomto
			// prípade znamená úspešnosť akcie.
			avatar.presuňDo(prechod);
			return true;
		}

		// Hodnota false informuje o neúspešnosti vykonania požadovanej akcie.
		return false;
	}

	/**
	 * Pokus o zobratie predmetu identifikovaného zadaným názvom. V prípade,
	 * že je akcia úspešá vráti metóda hodnotu true, inak false.
	 */
	public boolean zober(String názovPredmetu)
	{
		// Hľadá sa predmet v aktuálnej miestnosti:
		Predmet predmet = avatar.kdeSom().hľadajPredmet(názovPredmetu);

		if (null != predmet)
		{
			// Zobratie predmetu pozostáva z dvoch čiastkových akcií:
			// odstránenie predmetu zo zoznamu predmetov aktuálnej miestnosti
			// a vloženie predmetu do inventára avatara. Návratová hodnota
			// true signalizuje, že akcia bola úspešná (to de facto znamená,
			// že predmet bol nájdený v aktuálnej miestnosti, pretože žiadne
			// iné podmienky nie sú v súčasnosti overované).
			avatar.kdeSom().odstráňPredmet(predmet);
			avatar.vložPredmet(predmet);
			return true;
		}

		// Hodnota false informuje o neúspešnosti vykonania požadovanej akcie.
		return false;
	}

	/**
	 * Pokus o položenie predmetu identifikovaného zadaným názvom. V prípade,
	 * že je akcia úspešá vráti metóda hodnotu true, inak false.
	 */
	public boolean polož(String názovPredmetu)
	{
		// Hľadá sa predmet v inventári avatara:
		Predmet predmet = avatar.hľadajPredmet(názovPredmetu);

		if (null != predmet)
		{
			// Podobne ako zobratie predmetu, aj položenie pozostáva z dvoch
			// čiastkových akcií: odstránenie predmetu zo inventára
			// a vloženie do zoznamu predmetov aktuálnej miestnosti. Opäť,
			// návratová hodnota true signalizuje, že akcia bola úspešná.
			avatar.odstráňPredmet(predmet);
			avatar.kdeSom().vložPredmet(predmet);
			return true;
		}

		// Hodnota false informuje o neúspešnosti vykonania požadovanej akcie.
		return false;
	}


	// Rozpoznanie príkazu a spustenie príslušných akcií.
	public void spracujPríkaz()
	{
		// Definícia reťazca „príkaz“ (s hodnotou prevzatou zo vstupného
		// riadka) a deklarácia reťazca „parameter“.
		String príkaz = svet.prevezmiReťazec(), parameter;

		// Vypísanie príkazu (ozvena) a jeho prevod na malé písmená.
		vypíšText(modrá, príkaz);
		príkaz = príkaz.toLowerCase();

		// Ak zadaný príkaz začína slovom „choď“
		if (príkaz.startsWith("choď "))
		{
			// Prvých päť znakov (to jest aj medzera za slovom) sa preskočí
			// a zvyšok textu sa uloží do premennej „parameter“.
			parameter = príkaz.substring(5);

			// Pokus o prechod do zadanej miestnosti (to jest tej,
			// ktorej meno je v premennej „parameter“).
			if (choďDo(parameter))
			{
				// V prípade úspechu sa vymaže obrazovka
				// a vypíše sa nový stav hry.
				strop.vymažTexty();
				vypíšStav();
			}
			else
				vypíšText(červená, "Odtiaľto sa tam nedá ísť!", true);
		}
		else if (príkaz.startsWith("zober "))
		{
			// Principiálne rovnaké získanie parametra ako pri príkaze choď.
			parameter = príkaz.substring(6);

			// Pokus o zobratie predmetu
			if (zober(parameter))
				// V prípade úspechu sa vypíše pozitívna spätná väzba
				vypíšText(zelená, "Zobral si predmet: " +
					parameter + ".", true);
			else
				vypíšText(červená, "Nič také tu nevidíš!", true);
		}
		else if (príkaz.startsWith("polož "))
		{
			// Principiálne rovnaké získanie parametra ako pri príkaze choď.
			parameter = príkaz.substring(6);

			// Pokus o zobratie predmetu
			if (polož(parameter))
				// V prípade úspechu sa vypíše pozitívna spätná väzba
				vypíšText(zelená, "Položil si predmet: " +
					parameter + ".", true);
			else
				vypíšText(červená, "Nič také nemáš!", true);
		}
		else if (príkaz.startsWith("pomoc"))
		{
			vypíšText(zelená, "Pomocník");
			vypíšText(zelená, "K dispozícii sú nasledujúce príkazy: choď, " +
				"zober, polož, pomoc a koniec. Väčšina príkazov vyžaduje " +
				"zadanie parametra. Napríklad: choď ulica, zober kniha " +
				"a pod.", true);
		}
		else if (príkaz.startsWith("koniec"))
		{
			svet.koniec();
		}
		else
		{
			vypíšText(červená, "Neznámy príkaz!", true);
		}
	}


	/**
	 * Vlastná metóda na výpis textov. Prijíma farbu textu, text a argument
	 * určujúci, či sa má po vypísaní textu vynechať jeden prázdny riadok.
	 */
	public static void vypíšText(Farba farba, String text,
		boolean vynechajRiadok)
	{
		svet.farbaTextu(farba);
		svet.vypíšRiadok(text);
		if (vynechajRiadok) svet.vypíšRiadok();
	}

	/**
	 * Vlastná metóda na výpis textov. Správa sa ako predchádzajúca metóda,
	 * ale neprijíma argument určujúci, či sa má po vypísaní textu vynechať
	 * jeden prázdny riadok. Správa sa tak ako keby bolo zadané false.
	 */
	public static void vypíšText(Farba farba, String text)
	{
		vypíšText(farba, text, false);
	}

	/**
	 * Vlastná metóda na výpis textov. Správa sa ako predchádzajúca metóda,
	 * ale neprijíma argument určujúci farbu textu. Správa sa tak ako keby
	 * bola zadaná čierna.
	 */
	public static void vypíšText(String text)
	{
		vypíšText(čierna, text, false);
	}

	/**
	 * Vlastná metóda na výpis textov. Správa sa ako predchádzajúca metóda,
	 * ale namiesto argumentu typu String prijíma argument typu StringBuffer.
	 */
	public static void vypíšText(StringBuffer text)
	{
		vypíšText(čierna, text.toString(), false);
	}


	// Vytvorením novej inštancie adventúry sa automaticky spustí
	// konštruktor Adventúra(), ktorý inicializuje novú hru (vytvorí
	// prepojenia medzi miestnosťami a pod.).
	public static void main(String[] args)
	{
		new Adventúra();
	}
}
